VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsGSMTCSM"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' Autor: F. Schler (frank@activevb.de)
' Datum: 12/2022

Option Explicit

' ----==== Const ====----
Private Const S_OK As Long = &H0&
Private Const CC_STDCALL As Long = &H4&

Private Const WindowsMediaControlGlobalSystemMediaTransportControlsSessionManager As String = "Windows.Media.Control.GlobalSystemMediaTransportControlsSessionManager"
Private Const IID_IGlobalSystemMediaTransportControlsSessionManager As String = "{cace8eac-e86e-504a-ab31-5ff8ff1bce49}"
Private Const IID_IGlobalSystemMediaTransportControlsSessionManagerStatics As String = "{2050c4ee-11a0-57de-aed7-c97c70338245}"

Private Const IID_IClosable As String = "{30d5a829-7fa4-4026-83bb-d75bae4ea99e}"
Private Const IID_IAsyncInfo As String = "{00000036-0000-0000-c000-000000000046}"

' ----==== Enums ====----
Private Enum vtb_Interfaces
    
    ' IUnknown
    IUnknown_QueryInterface = 0
    IUnknown_Release = 2

    ' IInspectable
    IInspectable_GetIids = 3
    IInspectable_GetRuntimeClassName = 4
    IInspectable_GetTrustLevel = 5
    
    ' IAsyncInfo
    IAsyncInfo_GetStatus = 7
    IAsyncInfo_Close = 10

    ' IAsyncInterface
    IAsyncInterface_GetResults = 8
    
    ' IClosable
    IClosable_Close = 6
    
    ' IGlobalSystemMediaTransportControlsSessionManagerStatics
    IGlobalSystemMediaTransportControlsSessionManagerStatics_RequestAsync = 6
    
    ' IGlobalSystemMediaTransportControlsSessionManager
    IGlobalSystemMediaTransportControlsSessionManager_GetCurrentSession = 6
    IGlobalSystemMediaTransportControlsSessionManager_GetSessions = 7
    IGlobalSystemMediaTransportControlsSessionManager_AddCurrentSessionChanged = 8
    IGlobalSystemMediaTransportControlsSessionManager_RemoveCurrentSessionChanged = 9
    IGlobalSystemMediaTransportControlsSessionManager_AddSessionsChanged = 10
    IGlobalSystemMediaTransportControlsSessionManager_RemoveSessionsChanged = 11
                        
    ' IVectorView_IGlobalSystemMediaTransportControlsSession
    IVectorView_IGlobalSystemMediaTransportControlsSession_GetAt = 6
    IVectorView_IGlobalSystemMediaTransportControlsSession_GetSize = 7
                        
End Enum

Private Enum AsyncStatus
    Started = 0
    Completed = 1
    Canceled = 2
    Error = 3
End Enum

Private Enum TrustLevel
    BaseTrust = 0
    PartialTrust = 1
    FullTrust = 2
End Enum

' ----==== Types ====----
Private Type GUID
    data1 As Long
    data2 As Long
    data3 As Long
    data4 As Long
End Type

' ----==== Ole32.dll Declarations ====----
Private Declare Function CLSIDFromString Lib "Ole32.dll" ( _
                         ByVal pstring As Long, _
                         ByRef pCLSID As GUID) As Long
                         
Private Declare Function StringFromGUID Lib "Ole32.dll" _
                         Alias "StringFromGUID2" ( _
                         ByRef rguid As GUID, _
                         ByVal lpsz As Long, _
                         ByVal cchMax As Long) As Long
                         
Private Declare Sub CoTaskMemFree Lib "Ole32.dll" ( _
                    ByVal hMem As Long)
                         
' ----==== Oleaut32.dll Declarations ====----
Private Declare Function DispCallFunc Lib "Oleaut32.dll" ( _
                         ByVal pvInstance As Long, _
                         ByVal oVft As Long, _
                         ByVal cc As Long, _
                         ByVal vtReturn As VbVarType, _
                         ByVal cActuals As Long, _
                         ByVal prgvt As Any, _
                         ByVal prgpvarg As Any, _
                         ByRef pvargResult As Variant) As Long

' ----==== Kernel32.dll Declarations ====----
Private Declare Sub CopyMemory Lib "Kernel32.dll" _
                    Alias "RtlMoveMemory" ( _
                    ByRef pTo As Any, _
                    ByRef uFrom As Any, _
                    ByVal lSize As Long)

Private Declare Function lstrlenW Lib "Kernel32.dll" ( _
                         ByVal lpString As Long) As Long

' ----==== Combase.dll Declarations ====----
Private Declare Function WindowsCreateString Lib "Combase.dll" ( _
                         ByVal sourceString As Long, _
                         ByVal lenght As Long, _
                         ByRef hString As Long) As Long

Private Declare Function WindowsGetStringRawBuffer Lib "Combase.dll" ( _
                         ByVal sourceString As Long, _
                         ByRef length As Long) As Long

Private Declare Function WindowsDeleteString Lib "Combase.dll" ( _
                         ByVal sourceString As Long) As Long

Private Declare Function RoGetActivationFactory Lib "Combase.dll" ( _
                         ByVal activatableClassId As Long, _
                         ByRef riid As GUID, _
                         ByRef factory As Long) As Long

Private Declare Function RoActivateInstance Lib "Combase.dll" ( _
                         ByVal activatableClassId As Long, _
                         ByRef instance As Long) As Long

' ----==== Variablen ====----
Private m_pIGlobalSystemMediaTransportControlsSessionManager As Long
Private m_pIGlobalSystemMediaTransportControlsSessionManagerStatics As Long

Private m_pSessionsChanged As Long
Private m_lSessionsChangedToken As Long
Private m_pCurrentSessionsChanged As Long
Private m_lCurrentSessionsChangedToken As Long

' ----==== Events ====----
Public Event SessionsChanged(ByVal sender As Long, ByVal args As Long)
Public Event CurrentSessionsChanged(ByVal sender As Long, ByVal args As Long)

' ----==== Class ====----
Private Sub Class_Initialize()
    If GetActivationFactory(WindowsMediaControlGlobalSystemMediaTransportControlsSessionManager, _
                            IID_IGlobalSystemMediaTransportControlsSessionManagerStatics, _
                            m_pIGlobalSystemMediaTransportControlsSessionManagerStatics) Then
        If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManagerStatics, _
                  IGlobalSystemMediaTransportControlsSessionManagerStatics_RequestAsync, _
                  VarPtr(m_pIGlobalSystemMediaTransportControlsSessionManager)) = S_OK Then
            If WaitForAsync(m_pIGlobalSystemMediaTransportControlsSessionManager) Then
                m_pCurrentSessionsChanged = modITEH_CurrentSessionChanged.Create(Me)
                If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                          IGlobalSystemMediaTransportControlsSessionManager_AddCurrentSessionChanged, _
                          m_pCurrentSessionsChanged, VarPtr(m_lCurrentSessionsChangedToken)) = S_OK Then
                
                End If
                m_pSessionsChanged = modITEH_SessionsChanged.Create(Me)
                If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                          IGlobalSystemMediaTransportControlsSessionManager_AddSessionsChanged, _
                          m_pSessionsChanged, VarPtr(m_lSessionsChangedToken)) = S_OK Then
                
                End If
            End If
        End If
    End If
End Sub

Private Sub Class_Terminate()
    If m_pIGlobalSystemMediaTransportControlsSessionManager <> 0& Then
        If m_lSessionsChangedToken <> 0& Then
            If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                      IGlobalSystemMediaTransportControlsSessionManager_RemoveSessionsChanged, _
                      m_pSessionsChanged, m_lSessionsChangedToken) = S_OK Then
            End If
        End If
        If m_lCurrentSessionsChangedToken <> 0& Then
            If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                      IGlobalSystemMediaTransportControlsSessionManager_RemoveCurrentSessionChanged, _
                      m_pCurrentSessionsChanged, m_lCurrentSessionsChangedToken) = S_OK Then
            End If
        End If
        Call Release(m_pIGlobalSystemMediaTransportControlsSessionManager)
    End If
    Call Release(m_pIGlobalSystemMediaTransportControlsSessionManagerStatics)
End Sub

' ----==== Properties ====----
Public Property Get GetCurrentSession() As clsGSMTCS
    Dim Ret As clsGSMTCS
    If m_pIGlobalSystemMediaTransportControlsSessionManager <> 0& Then
        Dim pIGlobalSystemMediaTransportControlsSession As Long
        If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                  IGlobalSystemMediaTransportControlsSessionManager_GetCurrentSession, _
                  VarPtr(pIGlobalSystemMediaTransportControlsSession)) = S_OK Then
            Set Ret = New clsGSMTCS
            Ret.IGlobalSystemMediaTransportControlsSession = pIGlobalSystemMediaTransportControlsSession
        End If
    End If
    Set GetCurrentSession = Ret
End Property

Public Property Get GetSessionsCount() As Long
    Dim Ret As Long
    If m_pIGlobalSystemMediaTransportControlsSessionManager <> 0& Then
        Dim pIVectorView_IGlobalSystemMediaTransportControlsSession As Long
        If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                  IGlobalSystemMediaTransportControlsSessionManager_GetSessions, _
                  VarPtr(pIVectorView_IGlobalSystemMediaTransportControlsSession)) = S_OK Then
            Dim Value As Long
            If Invoke(pIVectorView_IGlobalSystemMediaTransportControlsSession, _
                      IVectorView_IGlobalSystemMediaTransportControlsSession_GetSize, _
                      VarPtr(Value)) = S_OK Then
                Ret = Value
            End If
            Call Release(pIVectorView_IGlobalSystemMediaTransportControlsSession)
        End If
    End If
    GetSessionsCount = Ret
End Property

Public Property Get GetSessionByIndex(ByVal Index As Long) As clsGSMTCS
    Dim Ret As clsGSMTCS
    If m_pIGlobalSystemMediaTransportControlsSessionManager <> 0& Then
        If Index >= 0 And Index < Me.GetSessionsCount Then
        Dim pIVectorView_IGlobalSystemMediaTransportControlsSession As Long
            If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                      IGlobalSystemMediaTransportControlsSessionManager_GetSessions, _
                      VarPtr(pIVectorView_IGlobalSystemMediaTransportControlsSession)) = S_OK Then
                Dim pIGlobalSystemMediaTransportControlsSession As Long
                If Invoke(pIVectorView_IGlobalSystemMediaTransportControlsSession, _
                          IVectorView_IGlobalSystemMediaTransportControlsSession_GetAt, _
                          Index, VarPtr(pIGlobalSystemMediaTransportControlsSession)) = S_OK Then
                    Set Ret = New clsGSMTCS
                    Ret.IGlobalSystemMediaTransportControlsSession = pIGlobalSystemMediaTransportControlsSession
                End If
                Call Release(pIVectorView_IGlobalSystemMediaTransportControlsSession)
            End If
        End If
    End If
    Set GetSessionByIndex = Ret
End Property

Public Property Get GetSessions() As clsGSMTCS()
    Dim Ret() As clsGSMTCS
    If m_pIGlobalSystemMediaTransportControlsSessionManager <> 0& Then
        Dim pIVectorView_IGlobalSystemMediaTransportControlsSession As Long
        If Invoke(m_pIGlobalSystemMediaTransportControlsSessionManager, _
                  IGlobalSystemMediaTransportControlsSessionManager_GetSessions, _
                  VarPtr(pIVectorView_IGlobalSystemMediaTransportControlsSession)) = S_OK Then
            Dim lngItems As Long
            If Invoke(pIVectorView_IGlobalSystemMediaTransportControlsSession, _
                      IVectorView_IGlobalSystemMediaTransportControlsSession_GetSize, _
                      VarPtr(lngItems)) = S_OK Then
                If lngItems > 0& Then
                    ReDim Ret(lngItems - 1)
                    Dim lngItem As Long
                    For lngItem = 0 To lngItems - 1
                        Dim pIGlobalSystemMediaTransportControlsSession As Long
                        If Invoke(pIVectorView_IGlobalSystemMediaTransportControlsSession, _
                                  IVectorView_IGlobalSystemMediaTransportControlsSession_GetAt, _
                                  lngItem, VarPtr(pIGlobalSystemMediaTransportControlsSession)) = S_OK Then
                            Set Ret(lngItem) = New clsGSMTCS
                            Ret(lngItem).IGlobalSystemMediaTransportControlsSession = pIGlobalSystemMediaTransportControlsSession
                        End If
                    Next
                End If
            End If
            Call Release(pIVectorView_IGlobalSystemMediaTransportControlsSession)
        End If
    End If
    GetSessions = Ret
End Property

' ----==== Subs & Functions ====----
Public Sub Invoke_SessionsChanged(ByVal sender As Long, _
                                  ByVal args As Long)
    Debug.Print "SessionsChanged", sender, args
    RaiseEvent SessionsChanged(sender, args)
End Sub

Public Sub Invoke_CurrentSessionsChanged(ByVal sender As Long, _
                                         ByVal args As Long)
    Debug.Print "CurrentSessionsChanged", sender, args
    RaiseEvent CurrentSessionsChanged(sender, args)
End Sub

Private Function GetActivationFactory(ByVal ClassName As String, _
                                      ByVal IID As String, _
                                      ByRef pFactory As Long) As Boolean
    Dim bolRet As Boolean
    bolRet = False
    Dim hString As Long
    If WindowsCreateString(StrPtr(ClassName), Len(ClassName), hString) = S_OK Then
        If hString <> 0& Then
            If RoGetActivationFactory(hString, Str2Guid(IID), pFactory) = S_OK Then bolRet = True
            Call WindowsDeleteString(hString)
        End If
    End If
    GetActivationFactory = bolRet
End Function

Private Function GetActivateInstance(ByVal ClassName As String, _
                                     ByVal IID As String, _
                                     ByRef pInstance As Long) As Boolean
    Dim bolRet As Boolean
    bolRet = False
    Dim hString As Long
    If WindowsCreateString(StrPtr(ClassName), Len(ClassName), hString) = S_OK Then
        If hString <> 0& Then
            Dim pIInspectable As Long
            If RoActivateInstance(hString, pIInspectable) = S_OK Then
                If pIInspectable <> 0& Then
                    If Invoke(pIInspectable, IUnknown_QueryInterface, _
                              VarPtr(Str2Guid(IID)), VarPtr(pInstance)) = S_OK Then
                        If pInstance <> 0& Then bolRet = True
                    End If
                    Call Release(pIInspectable)
                End If
            End If
            Call WindowsDeleteString(hString)
        End If
    End If
    GetActivateInstance = bolRet
End Function

Private Function WaitForAsync(ByRef pInterface As Long) As Boolean
    Dim bolRet As Boolean
    bolRet = False
    If pInterface <> 0& Then
        Dim pIAsyncInfo As Long
        If Invoke(pInterface, IUnknown_QueryInterface, _
                  VarPtr(Str2Guid(IID_IAsyncInfo)), _
                  VarPtr(pIAsyncInfo)) = S_OK Then
            Dim eStatus As AsyncStatus
            Do
                If Invoke(pIAsyncInfo, IAsyncInfo_GetStatus, _
                          VarPtr(eStatus)) = S_OK Then DoEvents
            Loop While eStatus = Started
            If eStatus = Completed Then
                bolRet = True
            Else
                Call Invoke(pIAsyncInfo, IAsyncInfo_Close)
            End If
            Call Release(pIAsyncInfo)
            If bolRet Then
                Dim pRet As Long
                If Invoke(pInterface, IAsyncInterface_GetResults, VarPtr(pRet)) = S_OK Then
                    Call Release(pInterface)
                    pInterface = pRet
                End If
            Else
                Call Release(pInterface)
            End If
        End If
    End If
    WaitForAsync = bolRet
End Function

Private Sub Release(ByRef pInterface As Long)
    Dim lngRet As Long
    If pInterface <> 0& Then
        Call Invoke(pInterface, IUnknown_Release)
        pInterface = 0&
    End If
End Sub

Private Function CloseAndRelease(ByRef pInterface As Long) As Long
    Dim lngRet As Long
    If pInterface <> 0& Then
        Dim pIClosable As Long
        If Invoke(pInterface, IUnknown_QueryInterface, _
                  VarPtr(Str2Guid(IID_IClosable)), _
                  VarPtr(pIClosable)) = S_OK Then
            Call Invoke(pIClosable, IClosable_Close)
            Call Release(pIClosable)
            Call Release(pInterface)
            pInterface = 0&
        End If
    End If
    CloseAndRelease = lngRet
End Function

Private Sub GetInspectableInfo(ByVal pInspectableInterface As Long, _
                               ByVal strInterface As String)
    If pInspectableInterface <> 0& Then
        Debug.Print "Inspectable Info for: " & strInterface
        Debug.Print "RuntimeClassName = " & GetInspectable_GetRuntimeClassName(pInspectableInterface)
        Dim eTrustLevel As TrustLevel
        eTrustLevel = GetInspectable_GetTrustLevel(pInspectableInterface)
        Select Case eTrustLevel
            Case TrustLevel.BaseTrust
                Debug.Print "TrustLevel = BaseTrust"
            Case TrustLevel.FullTrust
                Debug.Print "TrustLevel = FullTrust"
            Case TrustLevel.PartialTrust
                Debug.Print "TrustLevel = PartialTrust"
            Case Else
                Debug.Print "TrustLevel = UnknownTrust"
        End Select
        Dim tIIDs() As GUID
        Dim lngIIDsCount As Long
        lngIIDsCount = GetInspectable_GetIIDs(pInspectableInterface, tIIDs)
        Debug.Print "AviableIIDsCount = " & CStr(lngIIDsCount)
        If lngIIDsCount > 0 Then
            Dim lngIIDsItem As Long
            For lngIIDsItem = 0 To lngIIDsCount - 1
                Debug.Print "    AviableIID " & CStr(lngIIDsItem + 1) & _
                            " = " & Guid2Str(tIIDs(lngIIDsItem))
            Next
        End If
        Debug.Print String(60, "-")
    End If
End Sub

Private Function GetInspectable_GetIIDs(ByVal pInterface As Long, _
                                        ByRef tIIDs() As GUID) As Long
    Dim lngIIDCount As Long
    If pInterface <> 0& Then
        Dim pIIDs As Long
        If Invoke(pInterface, IInspectable_GetIids, _
                  VarPtr(lngIIDCount), VarPtr(pIIDs)) = S_OK Then
            If lngIIDCount > 0 Then
                Dim lngBytes As Long
                Dim lngIIDItem As Long
                ReDim tIIDs(lngIIDCount - 1)
                For lngIIDItem = 0 To lngIIDCount - 1
                    lngBytes = Len(tIIDs(lngIIDItem))
                    Call CopyMemory(tIIDs(lngIIDItem), _
                                    ByVal pIIDs + (lngBytes * lngIIDItem), _
                                    lngBytes)
                Next
            End If
        End If
    End If
    GetInspectable_GetIIDs = lngIIDCount
End Function

Private Function GetInspectable_GetRuntimeClassName(ByVal pInterface As Long) As String
    Dim strRet As String
    If pInterface <> 0& Then
        Dim hString As Long
        If Invoke(pInterface, IInspectable_GetRuntimeClassName, _
                  VarPtr(hString)) = S_OK Then
            strRet = Ptr2Str(WindowsGetStringRawBuffer(hString, 0))
            Call WindowsDeleteString(hString)
        End If
    End If
    GetInspectable_GetRuntimeClassName = strRet
End Function

Private Function GetInspectable_GetTrustLevel(ByVal pInterface As Long) As TrustLevel
    Dim eRet As TrustLevel
    If pInterface <> 0& Then
        Dim lngValue As Long
        If Invoke(pInterface, IInspectable_GetTrustLevel, _
                  VarPtr(lngValue)) = S_OK Then
            eRet = lngValue
        End If
    End If
    GetInspectable_GetTrustLevel = eRet
End Function

Private Function Invoke(ByVal pInterface As Long, _
                        ByVal vtb As vtb_Interfaces, _
                        ParamArray var()) As Variant
    If pInterface <> 0& Then
        Invoke = OleInvoke(pInterface, vtb, var)
    End If
End Function

Private Function OleInvoke(ByVal pInterface As Long, _
                           ByVal lngCmd As Long, _
                           ParamArray aParam()) As Variant
    If pInterface <> 0& Then
        If UBound(aParam) >= 0& Then
            Dim varParam As Variant
            varParam = aParam
            If IsArray(varParam) Then varParam = varParam(0)
            Dim lngCount As Long
            lngCount = UBound(varParam)
            Dim lngItem As Long
            Dim olePtr(10) As Long
            Dim oleTyp(10) As Integer
            For lngItem = 0& To lngCount
                oleTyp(lngItem) = VarType(varParam(lngItem))
                olePtr(lngItem) = VarPtr(varParam(lngItem))
            Next
        End If
        Dim varRet As Variant
        If DispCallFunc(pInterface, lngCmd * 4, CC_STDCALL, _
                        vbLong, lngItem, VarPtr(oleTyp(0)), _
                        VarPtr(olePtr(0)), varRet) <> S_OK Then
            Debug.Print "Fehler beim Aufrufen der Interface-Funktion!"
        End If

'        If varRet <> S_OK Then
'            Debug.Print "0x" & Hex$(varRet)
'        End If
    End If
    OleInvoke = varRet
End Function

Private Function Str2Guid(ByVal strGUID As String) As GUID
    Call CLSIDFromString(StrPtr(strGUID), Str2Guid)
End Function

Private Function Guid2Str(ByRef tGUID As GUID) As String
    Dim lngRet As Long
    Dim strGUID As String
    strGUID = String(40, 0)
    lngRet = StringFromGUID(tGUID, StrPtr(strGUID), Len(strGUID))
    If lngRet > 0 Then
        strGUID = Left$(strGUID, lngRet - 1)
    Else
        strGUID = vbNullString
    End If
    Guid2Str = strGUID
End Function

Private Function Ptr2Str(ByVal lpStr As Long) As String
    Dim lngLen As Long
    Dim bytBuffer() As Byte
    lngLen = lstrlenW(lpStr) * 2
    If lngLen > 0 Then
        ReDim bytBuffer(lngLen - 1)
        Call CopyMemory(bytBuffer(0), ByVal lpStr, lngLen)
        Call CoTaskMemFree(lpStr)
        Ptr2Str = bytBuffer
    End If
End Function
